//
//  LNSQTMovie.m
//  QTPlay
//
//  Created by Mark Alldritt on 24/09/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "LNSQTMovie.h"
#import <Carbon/Carbon.h>


NSString* LNSQTMoviePlayingTimeDidChangeNotification = @"LNSQTMoviePlayingTimeDidChangeNotification";


pascal Boolean MyActionFilter(MovieController mc, short action, void* params, long refCon)
{
	//	QTMovie does not post QTMovieTimeDidChangeNotification notifications while a movie is playing.  This
	//	specialization of QTMovie posts LNSQTMoviePlayingTimeDidChangeNotification notifications that other 
	//	objects can see current time changes resulting from movie playback.

    LNSQTMovie *movie = (LNSQTMovie*) refCon;
    
	switch (action) 
	{
	// handle idle events
	case mcActionIdle:
		if ([movie rate] != 0.0)
			[[NSNotificationCenter defaultCenter] postNotificationName:LNSQTMoviePlayingTimeDidChangeNotification
																object:movie];
		break;            
    }
    
	return false;
}	


@interface LNSQTMovie (Private)

-(void) _installMovieIdleCallback;
-(void) _removeMovieIdleCallback;

@end


@implementation LNSQTMovie (Private)

-(void) _installMovieIdleCallback
{
	ComponentResult cr = noErr;
	
	// install a movie controller action filter procedure; note that the document window must be marked
	// as non-deferred (in Interface Builder) or else [self movieController] will return nil here
	MovieController mc = [self quickTimeMovieController];
	if (!mc) goto bail;
	 
	MCActionFilterWithRefConUPP upp = NewMCActionFilterWithRefConUPP(MyActionFilter);
	if (!upp) goto bail;

	cr = MCSetActionFilterWithRefCon(mc, upp, (long) self);
	DisposeMCActionFilterWithRefConUPP(upp);

bail:
	return;
}

-(void) _removeMovieIdleCallback
{
	// remove existing callback by passing nil for the callback parameter
	NSParameterAssert([self quickTimeMovieController]);
	MCSetActionFilterWithRefCon([self quickTimeMovieController], nil, (long) NULL);
}

@end


@implementation LNSQTMovie

- (id)initWithFile:(NSString *)fileName error:(NSError **)errorPtr
{
	if ((self = [super initWithFile:fileName error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithURL:(NSURL *)url error:(NSError **)errorPtr
{
	if ((self = [super initWithURL:url error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithDataReference:(QTDataReference *)dataReference error:(NSError **)errorPtr
{
	if ((self = [super initWithDataReference:dataReference error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithPasteboard:(NSPasteboard *)pasteboard error:(NSError **)errorPtr
{
	if ((self = [super initWithPasteboard:pasteboard error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithData:(NSData *)data error:(NSError **)errorPtr
{
	if ((self = [super initWithData:data error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithMovie:(QTMovie *)movie timeRange:(QTTimeRange)range error:(NSError **)errorPtr
{
	if ((self = [super initWithMovie:movie timeRange:range error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithQuickTimeMovie:(Movie)movie disposeWhenDone:(BOOL)dispose error:(NSError **)errorPtr
{
	if ((self = [super initWithQuickTimeMovie:movie disposeWhenDone:dispose error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (id)initWithAttributes:(NSDictionary *)attributes error:(NSError **)errorPtr
{
	if ((self = [super initWithAttributes:attributes error:errorPtr]))
	{
		[self _installMovieIdleCallback];
	}
	
	return self;
}

- (void) dealloc
{
	[self _removeMovieIdleCallback];
	[super dealloc];
}

@end
